using System;
using System.Data;
using System.Data.Common;
using System.Collections;
using System.ComponentModel;
using iAnywhere.Data.UltraLite;

namespace iAnywhere.Data.UltraLite.CustomDataSet
{
	/// <summary>
	/// Summary description for CustomTable.
	/// </summary>
	public class CustomDataTable : IList, ITypedList
	{
		private ULDataReader					_table;
		private ULCursorSchema					_schema;
		private PropertyDescriptorCollection	_propertyDescriptors;
		private MissingMappingAction			_missingMappingAction;
		private string							_tableName;
		private int								_currentRowPosition;
		private int								_numberOfRows;

		internal CustomDataTable(  )
		{
			// Default value
			_missingMappingAction = MissingMappingAction.Passthrough;
		}
		
		public void Fill( ULDataReader table ) 
		{
			Fill( table, null );
		}

		public void Fill( ULDataReader reader, DataTableMapping tm )
		{
			_table = reader;
			_schema = reader.Schema;
			if (tm != null) {		// Maps the table's name if appropriate
				_tableName = tm.DataSetTable;
			} 
			else {
				_tableName = _schema.Name;
			}
			InitFromTable( tm );	// Initializes members
		}

		~CustomDataTable()
		{
		}

		/* Fills the _propertyDescriptors.
		 * The MissingMappingAction is handled here.
		 */
		private void InitFromTable( DataTableMapping tm )
		{
			String colName;
			Type colType;
			_propertyDescriptors = new PropertyDescriptorCollection( null );
			
			_table.MoveFirst();
			_currentRowPosition = 0;
			_numberOfRows = (int) _table.RowCount;

			for ( short i=0; i<_schema.ColumnCount; i++ ) // Loop through each column
			{
				colName = _schema.GetColumnName( i );
				
				// Take care of column name mappings
				if( ( tm == null ) || ( ! tm.ColumnMappings.Contains( colName ) ) )
				{
					if( MissingMappingAction == MissingMappingAction.Error )
					{
						throw new Exception( "MISSING_COLUMNMAPPING_SOURCE:" + colName );
					}
					else if( MissingMappingAction == MissingMappingAction.Ignore )
					{	continue; // Skips the column
					}
					else if( MissingMappingAction == MissingMappingAction.Passthrough )
					{
						// Nothing needs to be done, since the colName has 
						// been to the _schema's colName.  That is, passthrough has
						// already occurred
					}
				}
				else
				{
					colName = tm.ColumnMappings[ colName ].DataSetColumn;
				}
				
				colType = ULDotNETBridge.GetNETType(_schema.GetColumnULDbType( i ));
				
				if( colType != null ) {
					_propertyDescriptors.Add( new CustomTableColumn(i, colName, colType) );
				} else {
					throw new Exception( "Data type not allowed");
				}
			}
		}
		// gets a new blank row that conforms to the specified column schema
		private CustomTableRow NewRow() {
			return new CustomTableRow( _schema.ColumnCount );
		}
		public string Name {
			get {
				return _tableName;
			}
		}

		// ITypedList method
		// retrieves the descriptors that allows the DataGrid to discover the schema of the table
		// and query about the data values. Each of the Property Descriptors is associated with
		// a particular column within the table. The DataGrid can query each descriptor about the value
		// contained within the column, given a particular row.
		PropertyDescriptorCollection ITypedList.GetItemProperties( 
				PropertyDescriptor [] listAccessors ) {
			return _propertyDescriptors;
		}
		// ITypedList method
		// gets the name of the UltraLite Table/ResultSet that this class encapsulates. This name is used to
		// identify the table when external manipulations are done using DataGrid methods.
		// ie. table styles.
		string ITypedList.GetListName( PropertyDescriptor[] listAccessors ) {
			return _tableName;
		}
		// IList method
		// indicates that the size of the table is fixed and cannot be changed
		public bool IsFixedSize {
			get {
				return true;
			}
		}

		// IList method
		// indicates that the table rows are read-only
		public bool IsReadOnly {
			get {
				return true;
			}
		}

		public MissingMappingAction MissingMappingAction {
			get {
				return _missingMappingAction;
			}
			set {
				_missingMappingAction = value;
			}
		}

		// IList method
		// allows the DataGrid to retrieve a particular row given the row number
		// the set method is not implemented since this table is read-only
		public object this[int index] {
			get {
				if( index >= _numberOfRows || index < 0 ) {
					throw new ArgumentOutOfRangeException();
				}

				// moves to the correct point within the table
				_table.MoveRelative( index - _currentRowPosition );
				_currentRowPosition = index;
		
				CustomTableRow row = NewRow();

				// populates the blank row with the correct values
				if( !ULDotNETBridge.GetCurrentRowData( _table, _schema, row ) ) {
					throw new Exception( "Data type not allowed" );
				}
				return row;
			}
			set {
				throw new NotSupportedException();
			}
		}
		// IList method
		// Tells the DataGrid about the actual size of the table. Even though, the rows
		// are retrieved on an as needed basis, the proper number of rows needs to be
		// known by the DataGrid. ie. to adjust the scroll bar
		public int Count {
			get {
				return _numberOfRows;
			}
		}

		// IList -> ICollection method
		// indicates that this class is not thread safe
		public bool IsSynchronized {
			get {
				return false;
			}
		}

		// Unsupported IList methods
	
		public int Add(object value) {
			throw new NotSupportedException();
		}
	
		public void Clear() {
			throw new NotSupportedException();
		}

		public bool Contains(object value) {
			throw new NotSupportedException();
		}

		public int IndexOf(object value) {
			throw new NotSupportedException();
		}

		public void Insert(int index, object value) {
			throw new NotSupportedException();
		}

		public void Remove(object value) {
			throw new NotSupportedException();
		}

		public void RemoveAt(int index) {
			throw new NotSupportedException();
		}

		public void CopyTo(Array array, int index) {
			throw new NotSupportedException();
		}

		public object SyncRoot {
			get {
				throw new NotSupportedException();
			}
		}

		public IEnumerator GetEnumerator() {
			throw new NotSupportedException();
		}
	}
}
